#!/bin/sh

# return json strings
ret_ng=0
ret_str="["  # start


function check_ip()
{
	local name=$1

	CHECK=$(echo ${!name} | egrep "^(([1-9]?[0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\.){3}([1-9]?[0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])$")

	if [ -z "${CHECK}" ]; then
		ret_ng=1  # error
		ret_str=$ret_str"{\"result\":\"$name\"},"
	fi
}

function check_blank()
{
	local name=$1

	if [ -z "${!name}" ]; then
		ret_ng=1  # error
		ret_str=$ret_str"{\"result\":\"$name\"},"
	fi
}

function check_blank_mac()
{
	local name="cam""$1""_mac"

	if [ -z "${!name}" ]; then
		ret_ng=1  # error
		ret_str=$ret_str"{\"result\":\"cam"$1"_ip\"},"
	fi
}

function check_min_len()
{
	local name=$1
	local len=$2
	local param=${!name}

	if [ -z "$param" ] || [ ${#param} -lt $len ]; then
		ret_ng=1  # error
		ret_str=$ret_str"{\"result\":\"$name\"},"
	fi
}

function check_max_len()
{
	local name=$1
	local len=$2
	local param=${!name}

	if [ -z "$param" ] || [ ${#param} -gt $len ]; then
		ret_ng=1  # error
		ret_str=$ret_str"{\"result\":\"$name\"},"
	fi
}


# check network ip address
result_net=""
function check_network()
{
	if [ -z $1 ] ||  [ -z $2 ]  ||  [ -z $3 ]  ||  [ -z $4 ]; then
		result_net="ng"
		return 0
	fi

	local ip1_1=`echo $1 | awk -F. '{print $1}'`
	local ip1_2=`echo $1 | awk -F. '{print $2}'`
	local ip1_3=`echo $1 | awk -F. '{print $3}'`
	local ip1_4=`echo $1 | awk -F. '{print $4}'`
	local msk1_1=`echo $2 | awk -F. '{print $1}'`
	local msk1_2=`echo $2 | awk -F. '{print $2}'`
	local msk1_3=`echo $2 | awk -F. '{print $3}'`
	local msk1_4=`echo $2 | awk -F. '{print $4}'`
	local ip2_1=`echo $3 | awk -F. '{print $1}'`
	local ip2_2=`echo $3 | awk -F. '{print $2}'`
	local ip2_3=`echo $3 | awk -F. '{print $3}'`
	local ip2_4=`echo $3 | awk -F. '{print $4}'`
	local msk2_1=`echo $4 | awk -F. '{print $1}'`
	local msk2_2=`echo $4 | awk -F. '{print $2}'`
	local msk2_3=`echo $4 | awk -F. '{print $3}'`
	local msk2_4=`echo $4 | awk -F. '{print $4}'`

	result_net="unmatch"

	if [ $ip1_1 -ne $ip2_1 ]; then
		return 0
	fi
	if [ $msk1_2 -eq 255 ] && [ $msk2_2 -eq 255 ] && [ $ip1_2 -ne $ip2_2 ]; then
		return 0
	fi
	if [ $msk1_3 -eq 255 ] && [ $msk2_3 -eq 255 ] && [ $ip1_3 -ne $ip2_3 ]; then
		return 0
	fi
	if [ $msk1_4 -eq 255 ] && [ $msk2_4 -eq 255 ] && [ $ip1_4 -ne $ip2_4 ]; then
		return 0
	fi

	result_net="match"
}

function check_br0_ip()
{
	if [ $inet_if = "LTE" ]; then
		check_network "192.168.1.100" "255.255.255.0" $br0_address $br0_netmask
	fi
	if [ $inet_dhcp = "STATIC" ]; then
		check_network $inet_address $inet_netmask $br0_address $br0_netmask
	fi

	if [ $result_net = "match" ]; then
		ret_ng=1  # error
		ret_str=$ret_str"{\"result\":\"br0_address\"},"
		ret_str=$ret_str"{\"result\":\"br0_netmask\"},"
	fi
}

function check_br0_dhcp()
{
	if [ "$use_dhcp" = "0" ]; then
		return 0
	fi

	check_network $dhcpd_range_start $br0_netmask $br0_address $br0_netmask
	if [ $result_net = "unmatch" ]; then
		ret_ng=1  # error
		ret_str=$ret_str"{\"result\":\"dhcpd_range_start\"},"
	fi

	check_network $dhcpd_range_end $br0_netmask $br0_address $br0_netmask
	if [ $result_net = "unmatch" ]; then
		ret_ng=1  # error
		ret_str=$ret_str"{\"result\":\"dhcpd_range_end\"},"
	fi

	check_network $dhcpd_range_start $br0_netmask $dhcpd_range_end $br0_netmask
	if [ $result_net = "unmatch" ]; then
		ret_ng=1  # error
		ret_str=$ret_str"{\"result\":\"dhcpd_range_start\"},"
		ret_str=$ret_str"{\"result\":\"dhcpd_range_end\"},"
	fi
}


#
# check all
#
function check_val()
{
	# lte
	if [ $inet_if = "LTE" ]; then
		check_blank "lte_apn"
		check_blank "lte_user"
		check_blank "lte_password"
		check_blank "lte_auth"
	fi

	# check ip address value
	if [ $inet_dhcp = "STATIC" ] ; then
		check_ip "inet_address"
		check_ip "inet_netmask"
		check_ip "inet_gateway"
		check_ip "inet_dns1"
		check_ip "inet_dns2"
	fi

	# check blank safie params
	if [ $use_safie = "1" ] && [ $auto_safie = "0" ]; then
		check_blank "cam_count"

		for n in `seq 1 $cam_count`; do
			str="cam""$n""_tk"
			check_blank ${str}
			str="cam""$n""_user"
			check_blank  ${str}
			str="cam""$n""_passwd"
			check_blank  ${str}
			check_blank_mac $n
			str="cam""$n""_ip"
			check_ip ${str}
		done
		check_blank "scr_tk"
	fi

	# wifi
	if [ $use_wifi = "1" ]; then
		check_max_len "wlan_ssid" 32
		check_min_len "wlan_psk" 8
		check_max_len "wlan_psk" 63
		check_blank "wlan_proto"
	fi
	if [ $wlan_mode = "AP" ]; then
		check_blank "wlan_channel"
	fi

	# easylink
	if [ $use_easyln = "1" ]; then
		for n in `seq 0 7`; do
			str="easyln""$n""_en"
			if [ ${!str} == "1" ]; then
				prm="easyln""$n""_ip"
				check_ip ${prm}
				prm="easyln""$n""_port"
				check_blank ${prm}
				prm="easyln""$n""_type"
				check_blank ${prm}
				prm="easyln""$n""_mapp"
				check_blank ${prm}
			fi
		done
	fi

	# port forward
	for n in `seq 0 9`; do
		str="pf""$n""_en"
		if [ ${!str} == "1" ]; then
			prm="pf""$n""_ip"
			check_ip ${prm}
			prm="pf""$n""_port"
			check_blank ${prm}
			prm="pf""$n""_type"
			check_blank ${prm}
			prm="pf""$n""_mapp"
			check_blank ${prm}
		fi
	done

#	# external input and email
#	if [ $sinp_enb_p2 = "mail" ] || [ $sinp_enb_p4 = "mail" ]; then
#		check_blank "sinp_addr_1"
#		check_blank "sinp_myaddr"
#		check_blank "sinp_smtp"
#		check_blank "sinp_user"
#		check_blank "sinp_passwd"
#	fi
#	if [ $sinp_enb_p2 = "safie" ] && [ $use_safie = "0" ]; then
#		ret_ng=1  # error
#		ret_str=$ret_str"{\"result\":\"sinp_enb_p2\"},"
#	fi
#	if [ $sinp_enb_p4 = "safie" ] && [ $use_safie = "0" ]; then
#		ret_ng=1  # error
#		ret_str=$ret_str"{\"result\":\"sinp_enb_p4\"},"
#	fi
#	if [ -z $sinp_enb_p2 ]; then
#		ret_ng=1  # error
#		ret_str=$ret_str"{\"result\":\"sinp_enb_p2\"},"
#	fi
#	if [ -z $sinp_enb_p4 ]; then
#		ret_ng=1  # error
#		ret_str=$ret_str"{\"result\":\"sinp_enb_p4\"},"
#	fi

	# lan
	check_ip "br0_address"
	check_ip "br0_netmask"
	if [ $use_dhcp = "1" ]; then
		check_ip "dhcpd_range_start"
		check_ip "dhcpd_range_end"
	fi

	# check br0 subnet
	check_br0_ip
	check_br0_dhcp

	# check blank input w/o ip address
	if [ $inet_if = "LTE" ] && [ $ddns_prov != "disable" ]; then
		check_blank "ddns_domain"
		check_blank "ddns_user"
		check_blank "ddns_passwd"
	fi

	check_blank "ntp_url"

	# Reboot params
	if [ $use_rb_ping = "1" ]; then
		check_blank "ping_dest"
		check_blank "ping_sec"
		check_blank "ping_nfail"
		check_blank "ping_nrboot"
	fi
}

infile="/var/tmp/.temp_in"
outfile="/var/tmp/.temp_out"
res=0

rm $infile
rm $outfile

if [ $REQUEST_METHOD = "POST" ]; then
	if [ "$CONTENT_LENGTH" -gt 0 ]; then
		cat /dev/stdin > $infile
	else
		exit 1
	fi
else
	exit 1
fi

# json -> env
json2env.pl < $infile > $outfile

if [ $? -ne 0 ]; then
	echo "[{\"result\":\"bad_count\"}]"
	rm $infile $outfile
	exit 1
fi

rm $infile


# load env
. $outfile

# strip non static param
tmp_cfg="/var/tmp/.temp_strip"

grep -v "new_passwd" $outfile | \
grep -v "cam._ip"  | \
grep -v "cam._mac" > $tmp_cfg


# ip address -> mac address
clist_file="/var/tmp/.clist"

cam1_mac=""
cam2_mac=""
cam3_mac=""
cam4_mac=""

while read line
do
	tmp_ip=`echo $line | awk '{print $1}'`
	tmp_mac=`echo $line | awk '{print $2}'`
	if [ "$tmp_ip" = "$cam1_ip" ]; then
		cam1_mac="\"$tmp_mac\""
	elif [ "$tmp_ip" = "$cam2_ip" ]; then
		cam2_mac="\"$tmp_mac\""
	elif [ "$tmp_ip" = "$cam3_ip" ]; then
		cam3_mac="\"$tmp_mac\""
	elif [ "$tmp_ip" = "$cam4_ip" ]; then
		cam4_mac="\"$tmp_mac\""
	fi
done < $clist_file

echo "cam1_mac="\"$cam1_mac\" >> $tmp_cfg
echo "cam2_mac="\"$cam2_mac\" >> $tmp_cfg
echo "cam3_mac="\"$cam3_mac\" >> $tmp_cfg
echo "cam4_mac="\"$cam4_mac\" >> $tmp_cfg


# check value and make err result by json text
check_val

# add end of json
ret_str=${ret_str:0:-1}
ret_str="$ret_str""]"


# rw mount
#sudo /bin/mount -o rw,remount /

# update login passwd
passwd_file="/www/pages/.lighttpdpassword"

if [ -n "${new_passwd}" ]; then
	echo "admin:$new_passwd" > $passwd_file
fi

# send response
if [ $ret_ng -eq 0 ]; then
	echo "[{\"result\":\"ok\"}]"
	rm /www/pages/user.cfg.temp
	cp $tmp_cfg /www/pages/user.cfg
	rm $outfile $tmp_cfg
	/bin/sync
	sleep 1

	sudo /sbin/reboot
else
	echo $ret_str
	cp $tmp_cfg /www/pages/user.cfg.temp
	rm $outfile $tmp_cfg
fi

# ro mount
/bin/sync
#sudo /bin/mount -o ro,remount /


